import javax.swing.undo.*;

/** ClearUndo class.
 * This class handles how the system stores data so that the undo/redo
 * functionality is available.  Some of the code was borrowed from
 * Team Matrix.
 */
public class ClearUndo extends AbstractUndoableEdit {    
    private int[] oldRowIndex;
    private int[] oldColIndex;
    private UMTableModel oldModel;
    private Object[][] oldStuff;
    
    /** ClearUndo constructor.
     * Creates the temporary table to hold the values of each cell before the clear
     * operation takes place.
     *
     * @param model the table model prior to the clear operation
     * @param rowIndex the rows that were cleared
     * @param colIndex the columns that were cleared
     */    
    public ClearUndo(UMTableModel model, int[] rowIndex, int[] colIndex ) {
        System.out.println("clear");
        oldModel = model;
        oldRowIndex = rowIndex;
        oldColIndex = colIndex;
        oldStuff = new Object[rowIndex.length][colIndex.length];
        
        for (int i = 0; i < rowIndex.length; i++)
            for (int j = 0; j < colIndex.length; j++)
                oldStuff[i][j] = ""+oldModel.getValueAt(rowIndex[i], colIndex[j])+"";
    }
    
    /** The actual undo function to put back the values of each cell.
     *
     * @throws CannotUndoException if undo is unavailable
     */    
    public void undo() throws CannotUndoException {
        for (int i = 0; i < oldRowIndex.length; i++)
            for (int j = 0; j < oldColIndex.length; j++)
                oldModel.setValueAt(oldStuff[i][j], oldRowIndex[i], oldColIndex[j]);
    }
    
    /** The actual redo function to clear the selected cells.
     *
     * @throws CannotRedoException if redo is unavailable
     */    
    public void redo() throws CannotRedoException {
        for (int i = 0; i < oldRowIndex.length; i++)
            for (int j = 0; j < oldColIndex.length; j++)
                oldModel.setValueAt("",oldRowIndex[i], oldColIndex[j]);
    }
    
    /** Determines if undo can occur.
     *
     * @return true to allow undo
     */    
    public boolean canUndo() { return true; }
    
    /** Determines if redo can occur.
     *
     * @return true to allow redo
     */    
    public boolean canRedo() { return true; }
    
    /** Tells what operation the undo/redo is for.
     *
     * @return the name of the operation that occured
     */    
    public String getPresentationName() { return "Clear"; }
}

