import java.awt.*;
import java.io.*;
import java.net.*;
import javax.swing.*;
import javax.swing.undo.*;
import java.util.*;
import java.awt.image.*;
import java.awt.event.*;
import java.awt.datatransfer.*;

/** This class provides the methods available from the drop-down
 * edit menu of the main spread sheet.
 */
public class EditMenuOp implements ActionListener, ClipboardOwner {
    UMSpreadSheet m_spreadsheet;
    private JTable table;
    private UMTableModel tablemodel;
    private Clipboard clipboard = null;
    private CellPoint copypt = new CellPoint(UMSpreadSheet.m_baseRow, UMSpreadSheet.m_baseCol);
    
    // serey
    private String findValue;
    private boolean matchCase = false;
    private boolean matchCell = false;
    
    final private static ImageIcon findIcon = UMSpreadSheet.getImageIcon("find32.gif");
    
    /** The edit menu constructor.
     * All the hotkey's key stroke are registered so that they work.
     *
     * @param spreadsheet the current spreadsheet model
     */    
    public EditMenuOp(UMSpreadSheet spreadsheet) {
        m_spreadsheet = spreadsheet;
        table = spreadsheet.getTable();
        
        // Identifying the cut KeyStroke user can modify this
        
        KeyStroke undo = KeyStroke.getKeyStroke(KeyEvent.VK_Z, ActionEvent.CTRL_MASK, false);
        KeyStroke redo = KeyStroke.getKeyStroke(KeyEvent.VK_Y, ActionEvent.CTRL_MASK, false);
        // to cut on some other Key combination.
        // ADDED BY AL
        KeyStroke cut = KeyStroke.getKeyStroke(KeyEvent.VK_X,ActionEvent.CTRL_MASK,false);
        
        // Identifying the copy KeyStroke user can modify this
        // to copy on some other Key combination.
        // ADDED BY AL
        KeyStroke copy = KeyStroke.getKeyStroke(KeyEvent.VK_C,ActionEvent.CTRL_MASK,false);
        
        // Identifying the Paste KeyStroke user can modify this
        // to copy on some other Key combination.
        // ADDED BY AL
        KeyStroke paste = KeyStroke.getKeyStroke(KeyEvent.VK_V,ActionEvent.CTRL_MASK,false);
        
        // Identifying the Clear KeyStroke user can modify this
        // to copy on some other Key combination.
        // ADDED BY AL
        KeyStroke clear = KeyStroke.getKeyStroke(KeyEvent.VK_DELETE, 0, false);
        
        // registers the keyboard action with the actionlistener
        // ADDED BY AL
        table.registerKeyboardAction(this, "Undo", undo, JComponent.WHEN_FOCUSED);
        table.registerKeyboardAction(this, "Redo", redo, JComponent.WHEN_FOCUSED);
        table.registerKeyboardAction(this,"Cut", cut, JComponent.WHEN_FOCUSED);
        table.registerKeyboardAction(this,"Copy", copy, JComponent.WHEN_FOCUSED);
        table.registerKeyboardAction(this,"Paste",paste, JComponent.WHEN_FOCUSED);
        table.registerKeyboardAction(this,"Clear",clear, JComponent.WHEN_FOCUSED);
        
        tablemodel = spreadsheet.getTableModel();
    }
    // ADDED BY AL
    /** This method handles the undo functionality.
     */    
    public void undo_MenuItemActionPerformed() {
        System.out.println("undo_menuitemactionperformed");
        // Add your handling code here:
        int i[] = table.getSelectedColumns();
        int j[] = table.getSelectedRows();
        
        m_spreadsheet.getUndoManager().undo();
        m_spreadsheet.refreshUndoRedo();
        
        //need to check for cases where rows/cols deleted!
        int newColCount = table.getColumnCount();
        int newRowCount = table.getRowCount();
        if (j[0] >= newRowCount)
            j[0] = newRowCount-1;
        if (j[j.length-1] >= newRowCount)
            j[j.length-1] = newRowCount-1;
        if (i[0] >= newColCount)
            i[0] = newColCount-1;
        if (i[i.length-1] >= newColCount)
            i[i.length-1] = newColCount-1;
        
        table.setRowSelectionInterval(j[0],j[j.length-1]);
        table.setColumnSelectionInterval(i[0],i[i.length-1]);
        System.out.println("undo_menuitemactionperformed exit");
    }
    // ADDED BY AL
    /** This method handles the redo functionality.
     */    
    public void redoActionPerformed() {
        // Add your handling code here:
        int i[] = table.getSelectedColumns();
        int j[] = table.getSelectedRows();
        
        m_spreadsheet.getUndoManager().redo();
        m_spreadsheet.refreshUndoRedo();
        
        //need to check for cases where rows/cols deleted!
        int newColCount = table.getColumnCount();
        int newRowCount = table.getRowCount();
        if (j[0] >= newRowCount)
            j[0] = newRowCount-1;
        if (j[j.length-1] >= newRowCount)
            j[j.length-1] = newRowCount-1;
        if (i[0] >= newColCount)
            i[0] = newColCount-1;
        if (i[i.length-1] >= newColCount)
            i[i.length-1] = newColCount-1;
        table.setRowSelectionInterval(j[0],j[j.length-1]);
        table.setColumnSelectionInterval(i[0],i[i.length-1]);
    }
    
    /** This method handles the cut operation, which is done by calling
     * the copy method because essentailly they do the same thing.
     */    
    public void cutOp() { Copy(true); }
    
    /** This method will copy a range of cells.
     *
     * @param clear boolean to determine if a clear range will occur
     */    
    private void Copy(boolean clear) {
        if(table.getSelectedRowCount() != 0) {
            
            UndoableEdit edit = new CutUndo(tablemodel, table.getSelectedRows(), table.getSelectedColumns());
            
            tablemodel.setModified(true);
            
            String text, simpleText;
            
            CellRange range = tablemodel.getSelection();
            
            int startRow = table.getSelectedRow();
            int startCol = table.getSelectedColumn();
            int numrows = table.getSelectedRowCount();
            int numcols = table.getSelectedColumnCount();
            
            copypt = new CellPoint(startRow, startCol);
            
            
            
            simpleText = tablemodel.toString1(range, true); // copies value only for pasting into other applications
            //simpleText = tablemodel.CelltoString(range, true);
            text = tablemodel.toString1(range, false); // copies font formats for pasting within SpreadSheet
            
            UMClipObj strsel=new UMClipObj(simpleText, text);
            
            clipboard = Toolkit.getDefaultToolkit().getSystemClipboard();
            
            if(clear)
                tablemodel.clearRange(range);
            
            try {
                clipboard.setContents(strsel, this);
            } catch(NullPointerException e) {
                clipboard.setContents(strsel, this);
            } catch (Exception e) {}
            
            m_spreadsheet.getUndoSupport().postEdit(edit);
        }
    }
    
    /** This will perform copy operation.  Flag is set to false to keep the original
     * data from being cleared.
     */
    public void copyOp(){ Copy(false); }
    
    /** This will perform paste operation.
     *
     * From previous team (SHOULD CONTINUE AFTER CUT HAS BEEN DONE)
     */
    public void pasteOp() { // REMOVED SwingUtilities.invokeLater(new Runnable() { public void run() { BY AL
        BufferedImage testclip = null;
        String fromClip = null;
        CellRange affected = null;
        int size[] = null;
        Object clipboardContents = null;
        
        if (table.getSelectedRowCount() != 0) {
            int startRow = table.getSelectedRow();
            int startCol = table.getSelectedColumn();
            int rowOff = startRow-copypt.getRow();
            int colOff = startCol-copypt.getCol();
            
            int endRow = 0, endCol = 0;
            int tabrows = table.getRowCount(), tabcols = table.getColumnCount();
            
            if (clipboard == null){
                System.out.println("Clipboard is null");
                return;
            }
            try {
              clipboardContents = clipboard.getContents(this);
              if (clipboardContents != null){
                 Transferable transferable = clipboard.getContents(this);
                 if (transferable.isDataFlavorSupported(DataFlavor.stringFlavor)){

                     if (transferable.getTransferData(DataFlavor.plainTextFlavor) instanceof Reader)
                        fromClip = transferable.getTransferData(DataFlavor.stringFlavor).toString();
                    else
                        fromClip = transferable.getTransferData(DataFlavor.plainTextFlavor).toString();
                } else
                    System.out.println("Clipboard data is unknown type");
              }
              if (fromClip == null)
                fromClip = "";

              size=getSize(fromClip);

            } catch(Exception e){
                System.out.println("Error in Paste");
                System.out.println("Error Msg: "+e.getMessage());
                e.printStackTrace(System.out);
            }
            
            tablemodel.setModified(true);
            endRow = Math.min(tabrows-1, startRow+size[0]-1);
            endCol = Math.min(tabcols-1, startCol+size[1]-1);
            affected = new CellRange(startRow, endRow, startCol, endCol);
            
            PasteUndo edit = new PasteUndo(table, tablemodel, startRow, endRow-startRow+1, startCol, endRow-startRow+1);// ADDED BY AL
            
            tablemodel.fromString1(fromClip, rowOff, colOff, affected);
            
            edit.addRedoEdit();// ADDED BY AL
            
            // notify the listeners
            m_spreadsheet.getUndoSupport().postEdit(edit); // ADDED BY AL
        }
    }
    
    // ADDED BY AL FOR PASTE SPECIAL
    /** This method handles the special paste functionality.  If the
     * user would rather paste the raw values instead of the formula
     * then paste special would be used.
     */    
    public void pasteSpOp() {
        String fromClip = null;
        CellRange affected = null;
        int size[] = null;
        Object clipboardContents = null;
        
        if (table.getSelectedRowCount() != 0) {
            int startRow = table.getSelectedRow();
            int startCol = table.getSelectedColumn();
            int rowOff = startRow-copypt.getRow();
            int colOff = startCol-copypt.getCol();
            
            int endRow = 0, endCol = 0;
            int tabrows = table.getRowCount(), tabcols = table.getColumnCount();
            
            if (clipboard == null){
                System.out.println("Clipboard is null");
                return;
            }
            try {
              clipboardContents = clipboard.getContents(this);
              if (clipboardContents != null){
                 Transferable transferable = clipboard.getContents(this);
                 if (transferable.isDataFlavorSupported(DataFlavor.stringFlavor)){

                     if (transferable.getTransferData(DataFlavor.plainTextFlavor) instanceof Reader)
                        fromClip = transferable.getTransferData(DataFlavor.stringFlavor).toString();
                    else
                        fromClip = transferable.getTransferData(DataFlavor.plainTextFlavor).toString();
                } else
                    System.out.println("Clipboard data is unknown type");
              }
              if (fromClip == null)
                fromClip = "";

              size=getSize(fromClip);

            } catch(Exception e){
                System.out.println("Error in Paste");
                System.out.println("Error Msg: "+e.getMessage());
                e.printStackTrace(System.out);
            }
            
            tablemodel.setModified(true);
            endRow = Math.min(tabrows-1, startRow+size[0]-1);
            endCol = Math.min(tabcols-1, startCol+size[1]-1);
            affected = new CellRange(startRow, endRow, startCol, endCol);
            
            PasteUndo edit = new PasteUndo(table, tablemodel, startRow, endRow-startRow+1, startCol, endRow-startRow+1);// ADDED BY AL
            
            tablemodel.fromString1("PS"+fromClip, rowOff, colOff, affected);
            
            edit.addRedoEdit();// ADDED BY AL
            
            // notify the listeners
            m_spreadsheet.getUndoSupport().postEdit(edit); // ADDED BY AL
        }
    }
    
    /** This gets the row/col size of the string coming from Clipboard.
     *
     * @param input  string input from the clipboard
     * @return  an array that contains the row/col size
     */
    public int [] getSize(String input) {
        BufferedReader in = new BufferedReader(new StringReader(input));
        String line;
        int rowcount=0;
        int colcount=0;
        try {
            while ((line = in.readLine()) != null) {
                rowcount++;
                // initialize new tokenizer on line with tab delimiter.
                //		tokenizer = new StringTokenizer(line, "\t");
                int index;
                int prev=0;
                
                // set col to 1 before each loop
                int col=0;
                
                while (true) {
                    index = line.indexOf('\t', prev);
                    prev = index+1;
                    // increment column number
                    col++;
                    
                    if(index == -1) break;
                }
                if (colcount < col)
                    colcount = col;
            }
        } catch (Exception e) { return null; }
        //	System.out.println(new CellPoint(rowcount, colcount));
        int a[] = new int[2];
        a[0] = rowcount;
        a[1] = colcount;
        
        return a;
    }
    
    /** This will perform clear operation.
     */
    public void clearOp() {
        if (table.getSelectedRowCount() != 0) {
            System.out.println("clearop");
            // ADDED BY AL
            UndoableEdit edit = new ClearUndo(tablemodel, table.getSelectedRows(), table.getSelectedColumns());
            
            CellRange range = new CellRange(table.getSelectedRows(), table.getSelectedColumns());
            tablemodel.setModified(true);
            tablemodel.clearRange(range);
            
            // ADDED BY AL
            m_spreadsheet.getUndoSupport().postEdit(edit);
            System.out.println("clearop exit");
        }
    }
    
    /** Returns the index of a font from the system font list
     *
     * @param fname  name of the font being requested
     * @return  return the index (location) of the font
     */
    public static int getFontIndex(String fname) {
        java.awt.GraphicsEnvironment ge = java.awt.GraphicsEnvironment.getLocalGraphicsEnvironment();
        String[] fontNames = ge.getAvailableFontFamilyNames();
        int index=-2;
        for (int x = 0; x < fontNames.length; x++) {
            if (fontNames[x].equals(fname)) {
                index = x;
                break;
            }
        }
        return index;
    }
    
    /** Debug statement to determine if the program has lost the access to the clipboard.
     *
     * @param clipboard  nothing
     * @param transferable  nothing
     */
    public void lostOwnership(Clipboard clipboard, Transferable transferable) {
        System.out.println("Lost ownership!!!");
    }
    
    // ADDED BY AL
    /** This method listens to the actions that are going on from
     * the edit menu.
     * @param e name of the operation that has occured
     */    
    public void actionPerformed(ActionEvent e) {
        if (e.getActionCommand().compareTo("Cut")==0)
            cutOp();
        else if (e.getActionCommand().compareTo("Copy")==0)
            copyOp();
        else if (e.getActionCommand().compareTo("Paste")==0)
            pasteOp();
        else if (e.getActionCommand().compareTo("Paste Special") == 0)
            pasteSpOp();
        else if (e.getActionCommand().compareTo("Undo") == 0)
            undo_MenuItemActionPerformed();
        else if (e.getActionCommand().compareTo("Redo") == 0)
            redoActionPerformed();
    }

    
    // added Serey
    // START
     /** 
     * Wrapper function to fill a range of cell with a user defined value
     *
     * @param newValue is true if it should require a new value (Find...)
     *                 is false if it already has a value (Find Next)
     */
    public void find(boolean newValue) {
        CellPoint start;
        String inputValue = null;
	//checks if anything is selected

	if (table.getSelectedRowCount() != 0) {
	    int x = table.getSelectedRow();
	    int y = table.getSelectedColumn();
	    // start from the next cell
	    if (!newValue)
		if (y < table.getColumnCount()-1)
		    y++;
		else {
		    y = 1;
		    x++;
		}
	    
	    start = new CellPoint(x, y);
	    
        }
	else {
	    // or start from the beginning
            start = new CellPoint(UMSpreadSheet.m_baseRow, UMSpreadSheet.m_baseCol);
        }
        
	if (newValue) {
	    // ask for new value
	    //String inputValue = SharpOptionPane.showInputDialog
		//(sharp, "Find: ", "Find", findIcon, findValue, 10);
	    FindDialog findDialog = new FindDialog(m_spreadsheet, findValue, matchCase, matchCell);
	    findDialog.pack();
	    findDialog.setLocationRelativeTo(m_spreadsheet);
	    findDialog.setVisible(true);

	    inputValue = findDialog.getString();
            System.out.println("in EditMenuOp, inputValue = " + inputValue);
	    /*	    
	    matchCase = findDialog.isCaseSensitive();
	    matchCell = findDialog.isCellMatching();

	    Debug.println("case sens : " + findDialog.isCaseSensitive());
	    Debug.println("match cell : " + findDialog.isCellMatching());
	    */
	    //if input is cancelled or nothing is entered 
	    //then don't change anything
	    if (inputValue == null || inputValue.length() == 0)
		return;
	    else {
		//setFindValue(inputValue);
		matchCase = findDialog.isCaseSensitive();
		matchCell = findDialog.isCellMatching();

		Debug.println("case sens : " + findDialog.isCaseSensitive());
		Debug.println("match cell : " + findDialog.isCellMatching());
	    }
		
	}
	else if (!hasFindValue())
	    return;

	// PH: CellPoint found = tablemodel.look(start, tablemodel.fieldParser(findValue), matchCase, matchCell);
	CellPoint found = tablemodel.look(start, inputValue, matchCase, matchCell);
	if (found != null) {
	    //System.out.println(found);
	    table.setColumnSelectionInterval(found.getCol(), 
					     found.getCol());
	    table.setRowSelectionInterval(found.getRow(),
					  found.getRow());
	    // set it visible
	    table.scrollRectToVisible
		(new Rectangle
		    (table.getCellRect(found.getRow(), found.getCol(), true)));
	}	
	else {
	    JOptionPane.showMessageDialog
		(m_spreadsheet, "UMSpreadSheet has finished the search and no more \"" + inputValue + "\" is found.",
		 "Find Completed",
		 JOptionPane.INFORMATION_MESSAGE, findIcon); 
	}
    }
    
    

    /** This will set a find value as a string.
     *
     * @param s the string to find
     */    
    private void setFindValue(String s) {
	findValue = s;
	m_spreadsheet.checkFindNextState();
    }
    
       /**
     * Whether a value has been searched by the user
     *
     * @return true if the findValue has been set
     */
    public boolean hasFindValue() {
	return findValue != null && findValue.length() != 0;
    }
    // END SEREY
}