import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.*;
import javax.swing.table.*;
import javax.swing.filechooser.*;
import java.util.*;
import java.awt.print.*;
import java.awt.image.*;

/** This class will implement all the functions for the file menu.
 *
 */
public class FileMenuOp implements ActionListener, Printable {
    JDialog m_dialog;
    Button ok,cancel;
    
    final static private int UNNAMED = 0;
    final static private int FILE = 1;
    final static private int DATABASE = 2;
    
    //private int openType = UNNAMED;
    
    private int maxNumPage = 1;
    private boolean columnOverflow;
    private int closeCancel = 0;
    
    //private StringTokenizer tokenizer;
    
    
    private UMSpreadSheet m_spreadsheet;
    private JTable m_table;
    private UMTableModel m_tableModel;
    private JFileChooser m_fileChooser;
    private File m_file;
    final private static String m_extname = ".ums";
    private static UMFileFilter m_filter;
    
    private PrintStream m_out;
    private BufferedReader m_in;
    private BufferedImage m_img;
    
    private int m_printOption = 0;
    
    class UMFileFilter extends javax.swing.filechooser.FileFilter {
        /** If a file should be accepted.
         * @param m_file The file to be accpted.
         * @return true if File is correct, false otherwise.
         */
        public boolean accept(File m_file) {
            if (m_file.isDirectory()) {
                return true;
            }
            
            return m_file.getName().endsWith(m_extname);
        }
        
        /** Returns the discription of file.
         * @return The discription as a string.
         */
        public String getDescription() {
            return "UMSpreadsheet (*.ums)";
        }
    }
    /** Public Constructor: FileMenuOp
     * Description: This constructs the FileMenuOp and set the parent
     *                   application
     *
     * @param spreadsheet The UMSpreadSheet application
     */
    public FileMenuOp(UMSpreadSheet spreadsheet) {
        m_spreadsheet = spreadsheet;
        m_table = spreadsheet.getTable();
        m_tableModel = spreadsheet.getTableModel();
        m_fileChooser = new JFileChooser(new File("."));
        m_filter = new UMFileFilter();
        m_fileChooser.addChoosableFileFilter(m_filter);
        
        m_table.registerKeyboardAction(this,"New", KeyStroke.getKeyStroke
        (KeyEvent.VK_N,ActionEvent.CTRL_MASK,false),  JComponent.WHEN_FOCUSED);
        
        m_table.registerKeyboardAction(this,"OpenFile", KeyStroke.getKeyStroke
        (KeyEvent.VK_O,ActionEvent.CTRL_MASK,false), JComponent.WHEN_FOCUSED);
        
        m_table.registerKeyboardAction(this,"Save", KeyStroke.getKeyStroke
        (KeyEvent.VK_S,ActionEvent.CTRL_MASK,false), JComponent.WHEN_FOCUSED);
        
        m_table.registerKeyboardAction(this,"Print", KeyStroke.getKeyStroke
        (KeyEvent.VK_P,ActionEvent.CTRL_MASK,false), JComponent.WHEN_FOCUSED);
        
        m_table.registerKeyboardAction(this,"Exit", KeyStroke.getKeyStroke
        (KeyEvent.VK_E,ActionEvent.CTRL_MASK,false), JComponent.WHEN_FOCUSED);
    }
    
    /** Confirm dialog box that queries whether user wishes to save current document or not.
     * @return true to save, false otherwise
     */
    public int queryForSave() {
        // show confirm dialog box from static JOptionPane method
        String filename = "Untitled";
        if (m_file != null)
            filename = m_file.getName();
        
        int choice =
        UMOptionPane.showOptionDialog
        (m_spreadsheet,
        "Do you want to save the changes you made to \""+filename+"\"?",
        "Save",
        JOptionPane.YES_NO_CANCEL_OPTION,
        JOptionPane.QUESTION_MESSAGE,
        null);
        // return user's choice
        return choice;
    }
    
    /** Confirm dialog box that queries whether user wishes to close current document or not.
     *
     * @return true to close, false otherwise
     */
    public int queryForClose() {
        // show confirm dialog box from static JOptionPane method
        int choice =
        UMOptionPane.showOptionDialog
        (m_spreadsheet,
        "Do you want to close the current spreadsheet?",
        "close",
        JOptionPane.YES_NO_CANCEL_OPTION,
        JOptionPane.QUESTION_MESSAGE,
        null);
        // return user's choice
        return choice;
    }
    
    /** This will perform new file operation.
     */
    public void newFile() {
        if (m_tableModel.isModified()) {
            //if the table has been modify then ask if the user would like to save
            int choice = queryForSave();
            if (choice == 0) {
                saveFile();
            }
            else if (choice == 2) {
                //if user has cancel the new command it doesn't do any but return the function
                return;
            }
        }
        //create a new table
        UMSpreadSheet newSpreadSheet = new UMSpreadSheet();
        //get the table model from the new table
        UMTableModel newTable = newSpreadSheet.getTableModel();
        //convert each empty cell to a string to text
        String text = newTable.ConvertToString();
        m_tableModel = m_spreadsheet.getTableModel();
        //reset the table from the string, text, it should set every
        //cell to an empty cell
        m_tableModel.CellfromString(text, 0, 0,
        new CellRange(UMSpreadSheet.m_baseRow,
        50,
        UMSpreadSheet.m_baseCol,
        50));
        //reset the modification flag back to false
        m_tableModel.setModified(false);
        //reset the title and the filename
        m_spreadsheet.setTitle("University of Maryland CMSC435 Spreadsheet");
        m_file = null;
    }
    
    /** This will perform open file operation.
     */
    public void openFile() {
        /*UMOptionPane.showMessageDialog(m_spreadsheet,
                                       "This is the open file operation",
                                       "UMSpreadSheet Info",
                                       JOptionPane.WARNING_MESSAGE,
                                       null);*/
        
        int choice = m_fileChooser.showOpenDialog(m_spreadsheet);
        System.out.println("I have choice " + choice);
        if(choice == JFileChooser.APPROVE_OPTION) {
            File selectedfile = m_fileChooser.getSelectedFile();
            m_fileChooser = new JFileChooser(selectedfile.getParentFile());
            m_fileChooser.addChoosableFileFilter(m_filter);
            
            openTableModel(selectedfile);
            //System.out.println("I'll open a file");
            
            //Newly added - JJ
            (m_spreadsheet.getFileHistory()).addFileToHistoryAndUpdate(selectedfile);
            
        }
        else {
            //System.out.println("I won't open a file");
        }
    }
    
    //Newly added - JJ
    /** This will perform open file operation specified.
     * @param inFile The file name to be opened.
     */
    public void openFile(File inFile) {
        File selectedfile = inFile;
        m_fileChooser = new JFileChooser(selectedfile.getParentFile());
        m_fileChooser.addChoosableFileFilter(m_filter);
        
        openTableModel(selectedfile);
        
        (m_spreadsheet.getFileHistory()).addFileToHistoryAndUpdate(selectedfile);
        
    }
    
    /** Opens table model data from file.
     * @param aFile file name from which to open
     */
    public void openTableModel(File aFile) {
        
        StringBuffer textBuf = new StringBuffer();
        String line;
        //String password = null;
        try {
            m_in =  new BufferedReader(new FileReader(aFile));
            while ((line = m_in.readLine()) != null) {
                textBuf.append(line);
                textBuf.append("\n");
            }
            
            String text = textBuf.toString();
            
            //m_spreadsheet.newTableModel(50, 50);
            m_spreadsheet.getTable();
            m_tableModel = m_spreadsheet.getTableModel();
            m_tableModel.CellfromString(text, 0, 0,
            new CellRange(UMSpreadSheet.m_baseRow,
            50,
            UMSpreadSheet.m_baseCol,
            50));
            /*m_tableModel.fromString1(text, 0, 0,
                                  new CellRange(UMSpreadSheet.m_baseRow,
                                                50,
                                                UMSpreadSheet.m_baseCol,
                                                50));*/
            
            m_file = aFile;
            
            String title = m_file.getName();
            if (title.endsWith(m_extname))
                title = title.substring(0, title.length()-4);
            
            m_spreadsheet.setTitle(title+" - UMSpreadsheet");
            
        } catch (FileNotFoundException e) {
            fileOpError("Open", "File \""+aFile.getName()+"\" not found!");
        } catch (IOException e) {
            fileOpError("Open", "I/O error in opening \""+aFile.getName()+"\"!");
        } catch (Exception e) {
        }
    }
    
    /** This will perform save file operation.
     */
    public void saveFile() {
        /*UMOptionPane.showMessageDialog(m_spreadsheet,
                                       "This is the save file operation",
                                       "UMSpreadSheet Info",
                                       JOptionPane.WARNING_MESSAGE,
                                       null);*/
        if(m_tableModel.isModified()) {
            if (m_file == null) { //prompt to save as file
                saveAsFile();
            }
            else
            {System.out.println("not null");
             //Just save without asking
             m_tableModel.setModified(false);
             saveTableModel();
             //Newly added - JJ
             (m_spreadsheet.getFileHistory()).addFileToHistoryAndUpdate(m_file);
             
            }
        }
        //SaveDialogWindow();
    }
    
    
    /** This will perform saveAs file operation.
     */
    public void saveAsFile() {
        //display the Save As dialog box
        int choice = m_fileChooser.showSaveDialog(m_spreadsheet);
        if(choice == JFileChooser.APPROVE_OPTION) {
            // get selected file to save to
            File selectedfile = m_fileChooser.getSelectedFile();
            
            m_fileChooser = new JFileChooser(selectedfile.getParentFile());
            m_fileChooser.addChoosableFileFilter(m_filter);
            
            // enforce the extension name
            String path;
            
            try {
                path = selectedfile.getCanonicalPath();
            }
            catch (IOException e) {
                fileOpError("Save", "Unexpected error!");
                return;
            }
            
            if (!path.endsWith(m_extname))
                selectedfile = new File(path+m_extname);
            
            // check the existence of the file
            if (selectedfile.exists()) {
                choice =
                UMOptionPane.showOptionDialog
                (m_spreadsheet,
                "File "+selectedfile.getName()+" already exists.\n\n"+
                "Do you want to overwrite it?\n\n",
                "Save",
                JOptionPane.YES_NO_OPTION,
                JOptionPane.WARNING_MESSAGE,
                null, 1);
                if (choice != JOptionPane.YES_OPTION) {   //if user choose not to overwrite the file then close
                    closeCancel = 1;
                    return;
                }
            }
            //save the table
            m_tableModel.setModified(false);
            saveTableModel(selectedfile);
            //Newly added - JJ
            (m_spreadsheet.getFileHistory()).addFileToHistoryAndUpdate(selectedfile);
            
            
        }
        else{
            closeCancel = 1;
        }
    }
    
    
    /** Saves table model to specified file cell by cell in tab-delimited format.
     *
     * @param aFile the file name to save to.
     */
    public void saveTableModel(File aFile) {
        
        try {
            
            // initialize output
            m_out = new PrintStream(new FileOutputStream(aFile));
            
            int rowCount = m_tableModel.getRowCount();
            int colCount = m_tableModel.getColumnCount();
            // print out number of columns and rows
            //	    out.println(rowCount+"\t"+colCount);
            
            m_out.print(m_tableModel.ConvertToString());
            m_out.flush();
            
            m_file = aFile;
            // set modified to false
            m_tableModel.setModified(false);
            
            String title = m_file.getName();
            if (title.endsWith(m_extname))
                title = title.substring(0, title.length()-4);
            m_spreadsheet.setTitle(title+" - UMSpreadsheet");
        } catch (FileNotFoundException e) {
            fileOpError("Save", "File \""+aFile.getName()+"\" cannot be created!");
        } catch (IOException e) {
            fileOpError("Save", "I/O error in saving \""+aFile.getName()+"\"!");
        }
    }
    
    
    /** Overloaded save function that takes the current filename  as default.
     */
    public void saveTableModel() {
        saveTableModel(m_file);
    }
    
    /** Use for printing the file that is being sent from terp manager.
     * @param file The file which is to be printed.
     */
    public void setFile( File file ) { this.m_file = file; }
    
    /** This will perform print file operation.
     */
    public void printFile() {
        m_printOption =1;
        Object[] options = {"Print Selected Area",
        "Print Entire Docuement",
        "Cancel"};
        int n = JOptionPane.showOptionDialog(m_spreadsheet,
        "Do you want to " +
        "print the Entire Document" +
        " or Print the selected area" +
        " in the Spreadsheet?",
        "Print Option",
        JOptionPane.YES_NO_CANCEL_OPTION,
        JOptionPane.QUESTION_MESSAGE,
        null,options,options[2]);
        if (n == 2 || n == -1)
            return;
        m_printOption = n;
        printData();
    }
    /**
     * Initiates print job by creating a <code>PrintJob</code> object,
     * and assigning the current <code>FileOp</code> object to it.
     * It then calls up the Print Dialog, and then if user clicks print
     * on the dialog, print() is called on the PrintJob object.  This call
     * also calls the print method for the FileOp object, which takes
     * care of the creation of a printable table
     */
    public void printData() {
        
        // create temporary object that contains this FileOp object
        // and reset columnOverflow to false
        FileMenuOp fileOp = this;
        columnOverflow = false;
        
        try {
            PrinterJob prnJob = PrinterJob.getPrinterJob();
            prnJob.setPrintable(fileOp);
            
            // opens print dialog, and if user doesn't cancel the dialog
            // it calls print()
            prnJob.print();
        }
        catch (PrinterException e) {
            e.printStackTrace();
            fileOpError("Print", "Printing error: "+e.toString());
        }
    }
    
    /** Main print method of FileOp that performs calculations for
     * dimensions of printed table, and writes the strings out to
     * the <code>Graphics</code> object that is responsible for
     * holding the printed data.  All parameters are called by Java
     * classes, specifically <code>PrintJob</code>, not explicitly in
     * our code
     *
     * @param pg Graphics object that holds the printed information
     * @param pageFormat PageFormat object that holds print dimensions
     * @param pageIndex number of current Page
     * @return status of current call of print method
     * @throws PrinterException Thrown if cannot print.
     */
    public int print(Graphics pg, PageFormat pageFormat,
    int pageIndex) throws PrinterException {
        
        
        // if pageIndex is beyond maximum page number, then
        // don't return
        // set Graphics object to printed cartesian origin
        pg.translate((int)pageFormat.getImageableX(),
        (int)pageFormat.getImageableY());
        
        // get printable width and height of page
        int wPage = 0;
        int hPage = 0;
        
        
        wPage = (int)pageFormat.getImageableWidth();
        hPage = (int)pageFormat.getImageableHeight();
        
        
        
        if (pageIndex == 0) {
            if ( m_printOption == 1)
                m_img =(new CellRange(0,50, 1, 50,
                m_tableModel)).getImage(true);
            else
                m_img = m_tableModel.getSelection().getImage(true);
        }
        int width = m_img.getWidth();
        int height = m_img.getHeight();
        int currH = 0;
        int currW = 0;
        BufferedImage img;
        for (int i = 1; i <= pageIndex; i++) {
            currW = Math.min(currW + wPage, width);
            if (currW == width) {
                currW = 0;
                currH = Math.min(currH + hPage, height);
            }
            if (currH == height)
                return NO_SUCH_PAGE;
            
        }
        
        img = m_img.getSubimage(currW, currH,
        Math.min(wPage, width - currW ),
        Math.min(hPage, height - currH));
        pg.drawImage(img, 0, 0, null);
        
        // explicit call on garbage collector to flush all temporary data
        // used, such as graphics objects
        System.gc();
        return PAGE_EXISTS;
    }
    
    
    /** This will perform exit file operation.
     * @param operation which exit operation to perform.
     */
    public void exitFile(int operation) {
        //reset the flag to close the file
        closeCancel = 0;
        //check if the table has been modified or not, if has then
        //ask if the user wants to save before closing the table
        if (m_tableModel.isModified()) {
            int choice = queryForSave();
            if (choice == 0) {
                saveFile();
            }
            else if (choice == 2) { //if user cancel operation just return
                return;
            }
        }
        //Exit isn't from the default operation
        if (closeCancel == 0) {
            m_spreadsheet.dispose();
            if (m_spreadsheet.m_isStandAlone)
                System.exit(0);
        }
    }
    
    
    /** This method is activated on the Keystrokes we are listening to
     * in this implementation. Here it listens for keystroke
     * ActionCommands.
     *
     * Without this listener, when we press certain keys the individual
     * cell will be activated into editing mode in addition to the
     * effect of the key accelerators we defined with menu items.
     * With this key listener, we avoid this side effect.
     * @param e The actioncommand to be completed.
     */
    public void actionPerformed(ActionEvent e) {
        if (e.getActionCommand().compareTo("New") == 0) {
            newFile();
        }
        else if (e.getActionCommand().compareTo("OpenFile") == 0) { // CHANGED "Open" -> "OpenFile"
            openFile();
        }
        else if (e.getActionCommand().compareTo("Save") == 0) {
            saveFile();
        }
        else if (e.getActionCommand().compareTo("Print") == 0) {
            printData();
        }
        else if (e.getActionCommand().compareTo("Exit") == 0) {
            m_spreadsheet.exit();
        }
    }
    /**
     * a function to display error messages
     * @param op the operation that caused this error
     * @param error the error message
     */
    private void fileOpError(String op, String error) {
        UMOptionPane.showMessageDialog(m_spreadsheet, error, op,
        JOptionPane.ERROR_MESSAGE, null);
        m_table.requestFocus();
    }
    
    /** Returns the file.
     * @return The File object.
     */
    public File getFile() {
        return m_file;
    }
}