/*
 * @(#)FindDialog.java
 *
 * $Id: FindDialog.java,v 1.3 2001/05/27 22:28:57 huaz Exp $
 *
 * Created on November 28, 2000, 02:12 AM
 */

import javax.swing.*;
import java.awt.*;
import javax.swing.BorderFactory;
import javax.swing.border.Border;
import java.awt.event.*;

/**
 * This class provides a find dialog.
 * User is prompted to choose find options.
 *
 * @author Serey Lay
 * @version 1.4
 * @see SharpDialog
 */
public class FindDialog extends SharpDialog {
    
    private JLabel label;
    private JFrame frame;
    final private JTextField textField = new JTextField(10);
    private JCheckBox caseSensitiveBox;
    private JCheckBox matchCellBox;
    private String typedText = null;
    private boolean caseSensitive = false;
    private boolean matchCell = false;
    final private ImageIcon findIcon = UMSpreadSheet.getImageIcon("find32.gif");
    
    /** Constructor for FindDialog that starts to create the find dialog
     *  which extends from SharpDialog
     * @param aFrame creates the frame
     * @param findValue the value to be found
     * @param mcase true if you want te match the case
     */
    public FindDialog(JFrame aFrame, String findValue, boolean mCase,
    boolean mCell) {
        super(aFrame, "Find", true);
        
        textField.setText(findValue);
        
        caseSensitiveBox = new JCheckBox("Match Case");
        caseSensitiveBox.setMnemonic(KeyEvent.VK_M);
        caseSensitiveBox.setSelected(mCase);
        
        matchCellBox = new JCheckBox("Match Entire Cell Only");
        matchCellBox.setMnemonic(KeyEvent.VK_E);
        matchCellBox.setSelected(mCell);
        
        JPanel box = new JPanel(new BorderLayout(0, 5));
        
        box.add(textField, BorderLayout.NORTH);
        box.add(caseSensitiveBox, BorderLayout.WEST);
        box.add(matchCellBox, BorderLayout.EAST);
        
        setOptionPane(box,
        JOptionPane.PLAIN_MESSAGE,
        JOptionPane.OK_CANCEL_OPTION,
        findIcon);
    }
    
    /** Checks what you want to find is case sensitive
     * @return true or false
     */
    public boolean isCaseSensitive(){
        return caseSensitive;
    }
    
    /** Checks if the cell matches
     * @return true or false
     */
    public boolean isCellMatching(){
        return matchCell;
    }
    
    /** Returns a string value of the text typed
     * @return text typed
     */ 
    public String getString(){
        return typedText;
    }
    
    /** Function to create the Find Button text field
     * 
     */
    protected void onOpen() {
        textField.selectAll();
        textField.requestFocus();
    }
    
    /** Returns a boolean that this is the object that you want to see
     * @return true always
     */
    protected boolean onOK() {
        typedText = textField.getText();
        caseSensitive = caseSensitiveBox.isSelected();
        matchCell = matchCellBox.isSelected();
        return true;
    }
    
    
    
}
