import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.text.*;

/** This class is used to only allow number input (int/float).
 * It extends JTextField and provides its own Document Filter.
 *
 * For simplicity, now it uses InputVerifier - huaz
 */
public class NumberField extends JTextField {
    
    // private Float number;
    // private String oldValue;
    // private Float number;
    /** a field to indicated that only positive numbers are allowed.
     */    
    private boolean positiveOnly;
    /** a field to indicate that only integers are allowed.
     */    
    private boolean integer;
    
    /** Construct a float-number only text field (include integers).
     *
     * @param columns the length of the text field
     */
    public NumberField(int columns) {
        super(columns);
        setDocument(new NumberFilterDocument());
        // setInputVerifier(new NumberVerifier());
    }
    
    /** Construct a float only text field.  Positive number only restrictions can be
     * set.
     *
     * @param columns the length of the text field
     * @param positive sets/resets the positive number restriction
     */
    public NumberField(int columns, boolean positive) {
        this(columns);
        this.positiveOnly = positive;
        setDocument(new NumberFilterDocument());
        // setInputVerifier(new NumberVerifier());
    }
    
    /** Construct a number only text field.  Positive number only restrictions or
     * integer only restrictions can be set.
     *
     * @param columns the length of the text field
     * @param positive sets/resets the positive number restriction
     * @param integer sets/resets the integer only restriction
     */
    public NumberField(int columns, boolean positive, boolean integer) {
        this(columns);
        this.positiveOnly = positive;
        this.integer = integer;
        setDocument(new NumberFilterDocument());
        //	setInputVerifier(new NumberVerifier());
    }
    
    /** Construct a number only text field.  Positive number only restrictions or
     * integer only restrictions can be set.
     *
     * @param positive sets/resets the positive number restriction
     * @param integer sets/resets the integer only restriction
     */
    public NumberField(boolean positive, boolean integer) {
        super();
        this.positiveOnly = positive;
        this.integer = integer;
        setDocument(new NumberFilterDocument());
        //setInputVerifier(new NumberVerifier());
    }
    
    /** Gets the input float number.
     *
     * @return Float object from the user's input
     */
    public Float getNumber() {
        try {
            return new Float(Float.parseFloat(getText()));
        }
        catch (Exception e) {
            return null;
        }
    }
    
    /** Sets the textfield to the specified float number.
     *
     * @param f a Float object
     */
    public void setNumber(Float f) {
        if (f == null)
            setText(null);
        else
            setText(f.toString());
    }
    
    /** Gets the input integer number.
     *
     * @return Integer object from the user's input
     */
    public Integer getInteger() {
        try {
            return new Integer(Integer.parseInt(getText()));
        }
        catch (Exception e) {
            return null;
        }
    }
    
    /** Sets the textfield to the specified integer number.
     *
     * @param i an Integer object
     */
    public void setInteger(Integer i) {
        if (i == null)
            setText(null);
        else
            setText(i.toString());
    }
    
    /** A class used to filter the user's input.
     */
    class NumberFilterDocument extends PlainDocument {
        /** private temp.
         */        
        private StringBuffer __scratchBuffer;
        
        /** Number Filter Document constructor
         */
        public NumberFilterDocument() {
            super();
            __scratchBuffer = new StringBuffer();
        }
        
        /** This function is called when user enters a string into the text field.
         *
         * @param offset location for input
         * @param text input entered by user
         * @param aset something
         * @throws BadLocationException bad location
         */
        public void insertString(int offset, String text, AttributeSet aset) throws BadLocationException {
            if (text == null)
                return;
            
            __scratchBuffer.setLength(0);
            
            // Reject all strings that cause the contents of the field not
            // to be a valid number (i.e., string representation of a double)
            try {
                __scratchBuffer.append(getText(0, getLength()));
                __scratchBuffer.insert(offset, text);
                // Kludge: Append a 0 so that leading decimal points
                // and signs will be accepted
                __scratchBuffer.append('0');
            } catch (BadLocationException ble) {
                ble.printStackTrace();
                return;
            } catch (StringIndexOutOfBoundsException sioobe) {
                sioobe.printStackTrace();
                return;
            }
            
            float value;
            try {
                if (integer)
                    value = Integer.parseInt(__scratchBuffer.toString());
                else
                    value = Float.parseFloat(__scratchBuffer.toString());
            } catch (NumberFormatException nfe) {
                // Resulting string will not be number, so reject it
                return;
            }
            
            if (positiveOnly && value < 0)
                return;
            
            super.insertString(offset, text, aset);
        }
    }
    /* from numberfield.java
class NumberVerifier extends InputVerifier {
     
    NumberVerifier() {
    super();
    }
     
    public boolean verify(JComponent input) {
        JTextField tf = (JTextField)input;
        String number = tf.getText();
        float value = 0;
        try {
            if (integer)
                value = Integer.parseInt(number);
            else
                value = Float.parseFloat(number);
        } catch(NumberFormatException nfe) {
            // Resulting string will not be number, so reject it
            return false;
        }
     
        if (positiveOnly && value < 0)
            return false;
     
        return true;
    }
}
     */
}