import java.awt.*;
import java.awt.event.*;

/** This class is used to create the splash screen
 *
 * @author Johnny John
 * @see Window
 */
public class Splash extends Window {
  private Image splashImage;
  private int imgWidth, imgHeight;
  private String imgName;
  private static final int BORDERSIZE = 0;
  private static final Color BORDERCOLOR = Color.blue;
  Toolkit tk;

  /** Constructor to create the splash screen
   *
   * @param Frame current frame
   * @param String the image name
   */
  public Splash(Frame f, String imgName) {
    super(f);
    this.imgName = imgName;
    tk = Toolkit.getDefaultToolkit();
    splashImage = loadSplashImage();
    showSplashScreen();
    f.addWindowListener(new WindowListener());
  }
  
  
  /** This will start to load the images onto the splash screen
   *
   * @returns an image of the splash screen
   */
  public Image loadSplashImage() {
    MediaTracker tracker = new MediaTracker(this);
    Image result;
    result = tk.getImage(imgName);
    tracker.addImage(result, 0);
    try {
      tracker.waitForAll();
      }
    catch (Exception e) {
      e.printStackTrace();
      }
    imgWidth = result.getWidth(this);
    imgHeight = result.getHeight(this);
    //System.out.println(imgWidth + " x " + imgHeight);
    return (result);
  }

  
  
  /** This will show the splash screen
   *
   */
  public void showSplashScreen() {
    Dimension screenSize = tk.getScreenSize();
    setBackground(Color.white);
    int w = imgWidth + (BORDERSIZE * 2);
    int h = imgHeight + (BORDERSIZE * 2);
    //int w = imgWidth;
    //int h = imgHeight;
    int x = (screenSize.width - w) /2;
    int y = (screenSize.height - h) /2;
    setBounds(x, y, w, h);
    setVisible(true);
  }
  
  
  /** Overrides the Window paint(Graphics) method
   *
   * @param Graphics the current graphics
   */
  public void paint(Graphics g) {
   g.drawImage(splashImage, BORDERSIZE, BORDERSIZE, imgWidth, imgHeight, this);
  }
  
  
  /** This class is used
   *
   */
  class WindowListener extends WindowAdapter {
      
        /** Overrides WindowAdapter's windowOpened(WindowEvent) method
         *
         * @param WindowEvent
         */
        public void windowOpened(WindowEvent we) {
            setVisible(false);
            dispose();
        }
  }
  
  
}