import javax.swing.*;
import javax.swing.border.*;
import java.awt.Color;
import java.awt.Rectangle;
import java.awt.Font;
import java.awt.Component;
import javax.swing.UIManager;

/**
 *  Class: UMCellEditor
 *  Description: This is the only editor for the UMTableModel
 *             because it is the only cell object editor.
 *             In the future it will read all of the settings
 *             for the cell in the spreadsheet and display them
 *             accordingly
 *
 * @see DefaultCellEditor
 * @see JTextField
 * @author Serey Lay
 * @version 1.0
 *
 */
public class UMCellEditor extends DefaultCellEditor {
    private JTextField m_textField;
    /**
     * Creates a new SharpCellEditor
     *
     * @param textField creats new JTextField
     */
    public UMCellEditor(final JTextField textField) {
        super(textField);
        this.m_textField = textField;
        /* To account for formula feature only need
         * to override the setValue method in EditorDelegate inner
         * class.
         */
        delegate = new EditorDelegate()	{
            public void setValue(Object value) {
                if (value instanceof UMCell) {
                    UMCell temp = (UMCell)value;
                    // when editing formula cell a string representation is displayed
                    if (temp.isFormula()) {
                        System.out.println("delegate");
                        textField.setText("=" + temp.getFormula().toString());
                    } else {
                        //otherwise it is just the normal string conversion
                        textField.setText(temp.getValue().toString());
                    }
                    
                } else {
                    //empty cells display nothing
                    textField.setText((value == null) ? "" : value.toString());
                }
                
            }
            
            public Object getCellEditorValue() {
                return textField.getText();
            }
        };
        
        textField.addActionListener(delegate);
    }
    
    /** Gets the component used by this editor
     *  @return JTextField used by this editor
     *  @see JTextField
     */
    public JTextField getTextField() { return m_textField; }
    
    /**
     * Implements javax.swing.table.TableCellRenderer
     * it supposed to initialize the editor for each
     * individual cell and return it.
     *
     * @param table Jtable java standard table
     * @param value value to assign to the cell at row, column
     * @param isSelected true if cell is selected
     * @param row the row of the cell to render
     * @param column the column of the cell to render
     * @return  Component
     * @see Component
     */
    public Component getTableCellEditorComponent(JTable table, Object value, boolean isSelected,
    int row, int column) {
        Object obj = table.getModel().getValueAt(row, column);
        if (obj instanceof UMCell) {
            UMCell cell = (UMCell) obj;
            Font font = cell.getFont();
            Color bColor = cell.getBackground();
            Color fColor = cell.getForeground();
            
            if (font == null) {
                font = table.getFont();
                cell.setFont(font);
            }
            if (bColor == null) {
                bColor = table.getBackground();
                cell.setBackground(bColor);
            }
            if (fColor == null) {
                fColor = table.getForeground();
                cell.setForeground(fColor);
            }
            m_textField.setForeground(fColor);
            m_textField.setBackground(bColor);
            m_textField.setFont(font);
        } else {
            m_textField.setForeground(table.getForeground());
            m_textField.setBackground(table.getBackground());
            m_textField.setFont(table.getFont());
        }
        setValue(value, isSelected, row, column);
        return m_textField;
    }
    
    /** Sets the string for the cell being edited to value.
     *
     * @param value the string value for this cell, if null empty string
     * @param isSelected whether the cell is selected.
     * @param row cell row
     * @param column cell column
     */
    protected void setValue(Object value, boolean isSelected, int row, int column) {
        if (value instanceof UMCell) {
            //we only care about the value, not the formula string
            UMCell temp = (UMCell)value;
            
            // getFormula
            if (temp.isFormula()) {
                System.out.println("setvalue");
                m_textField.setText("=" + temp.getFormula().toString());
            } else {
                Object data = temp.getValue();
                //value to display in table
                m_textField.setText((data == null) ? "" : data.toString());
            }
        } else
            m_textField.setText((value == null) ? "": value.toString());
    }
}