import javax.swing.*;
import javax.swing.table.*;
import javax.swing.border.*;
import javax.swing.event.*;
import javax.swing.undo.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;
import java.net.*;

/** This is the main class for the project it contains the main start method
 * and the main gui that is loaded on start up.
 *
 * Disclaimer: Some of the code and ideas were taken from the ShartTools project
 *            The previous project team integrated their code with the borrowed
 *            code.
 *
 * @version 2.0
 */
final public class UMSpreadSheet
extends JFrame implements ListSelectionListener, MouseListener, ActionListener, TerpOfficeAppInterface {
    // serey
    private JMenuItem findNextMenuItem;
    
    // ADDED BY AL FOR FORMULA BAR, UNDO, REDO
    private JTextField formulaField;
    private UndoManager undoManager= new UndoManager();
    private UndoableEditSupport undoSupport = new UndoableEditSupport();
    /** This method is required to correctly implement the undo/redo
     * functionality.
     *
     * @return a new undomanage
     */
    public UndoManager getUndoManager() { return undoManager; }
    /** This method is required to correctly implement the undo/redo
     * functionality.
     * @return a undo manager support
     */
    public UndoableEditSupport getUndoSupport() { return undoSupport; }
    
    /** Determines if the spreadsheet is a stand alone function.
     */
    public boolean m_isStandAlone = false;
    
    // the rest of the components
    /** an instance of the gui container.
     */
    private Container m_container;
    /** an instance of the gui main table.
     */
    private JTable m_table;
    /** an instance of the gui main scroll pane.
     */
    private JScrollPane m_scrollPane;
    /** an instance of the gui main tool bar.
     */
    private JToolBar m_toolBar;
    /** an instance of the gui main menu bar.
     */
    private JMenuBar m_menuBar;
    /** an instance of the gui main bar panel.
     */
    private JPanel m_barPanel;
    /** an instance of the gui main input pane.
     */
    private JPanel m_inputPane;
    /** the gui bold toggle button to indicate pressed/depressed.
     */
    public  JToggleButton boldButton;
    /** the gui italics toggle button to indicate pressed/depressed.
     */
    public  JToggleButton italicButton;
    
    /** current spreadsheet configuration file.
     */
    static private Config m_config;
    
    /** a copy of the current table model.
     */
    private UMTableModel m_tableModel;
    /** the current number of rows highlighted.
     */
    private ListSelectionModel m_rowSelectionModel;
    /** the current number of columns highlighted.
     */
    private ListSelectionModel m_columnSelectionModel;
    
    // these are used to access our global objects
    /** gets the current configuration file.
     *
     * @return current configuration file
     */
    static public Config getConfig() { return m_config; }
    
    /** gets the main main table.
     *
     * @return current main table
     */
    public JTable getTable() { return m_table; }
    /** gets the main table model.
     *
     * @return current table model
     */
    public UMTableModel getTableModel() { return m_tableModel; }
    /** gets the main tool bar.
     *
     * @return main tool bar
     */
    public JToolBar getToolBar() { return m_toolBar; }
    /** gets the main menu bar.
     *
     * @return main menu bar
     */
    public JMenuBar getJMenuBar() { return m_menuBar; }
    
    
    /** operations allowed within file tool bar.
     */
    private FileMenuOp m_fileMenuOp;
    /** operations allowed within edit tool bar.
     */
    private EditMenuOp m_editMenuOp;
    /** operations allowed within format tool bar.
     */
    private FormatMenuOp m_formatMenuOp;
    /** operations allowed within functions tool bar.
     */
    private FunctionsMenuOp m_functionsMenuOp;
    
    // serey
    private HelpMenuOp m_helpMenuOp; // new
    
    private GraphsMenuOp m_graphsMenuOp; //AMAR
    
    // initialize JMenu fields
    // menu headings
    /** the gui menu structure for file.
     */
    private JMenu m_fileMenu;
    /** the gui menu structure for edit.
     */
    private JMenu m_editMenu;
    /** the gui menu structure for format.
     */
    private JMenu m_formatMenu;
    
    // submenu headings
    private JMenu insertMenu;
    private JMenu deleteMenu;
    
    /** the gui menu structure for functions.
     */
    private JMenu m_functionsMenu;
    
    /** the gui menu structure for help.
     */
    private JMenu m_helpMenu; // new
    
    private JMenu m_graphsMenu; //AMAR
    
    /** the gui structure for new pop-up menus.
     */
    private PopupMenu m_popup;
    
    // Tool bar selections
    /** the gui dropdown box for the list of fonts.
     */
    public JComboBox m_fontList;
    /** the gui dropdown box for the list of font sizes.
     */
    public JComboBox m_fontSizeList;
    
    
    /** Tool bar selections for FUNCTION serey
     */
    public JComboBox m_functionList;
    
    // recources
    /** the shortcut image for the new icon.
     */
    final private ImageIcon newIcon = getImageIcon("new.gif");
    /** the shortcut image for the open icon.
     */
    final private ImageIcon openIcon = getImageIcon("open.gif");
    /** the shortcut image for the save icon.
     */
    final private ImageIcon saveIcon = getImageIcon("save.gif");
    /** the shortcut image for the print icon.
     */
    final private ImageIcon printIcon = getImageIcon("print.gif");
    /** the shortcut image for the cut icon.
     */
    final private ImageIcon cutIcon = getImageIcon("cut.gif");
    /** the shortcut image for the copy icon.
     */
    final private ImageIcon copyIcon = getImageIcon("copy.gif");
    /** the shortcut image for the paste icon.
     */
    final private ImageIcon pasteIcon = getImageIcon("paste.gif");
    /** the shortcut image for the bold icon.
     */
    final private ImageIcon boldIcon = getImageIcon("bold.gif");
    /** the shortcut image for the italics icon.
     */
    final private ImageIcon italicIcon = getImageIcon("italic.gif");
    
    // serey
    final private ImageIcon fcolorIcon = getImageIcon("fcolor.gif");
    final private ImageIcon bcolorIcon = getImageIcon("bcolor.gif");
    final private ImageIcon formatIcon = getImageIcon("format.gif");
    
    // serey
    // this sets where you want to place the point
    static private int x = 50;
    static private int y = 50;
    
    // this sets the dim of rows and columns
    static private int w = 500;
    static private int h = 500;
    
    
    /** starting index for row.
     */
    public static int m_baseRow = 0;
    /** starting index for columns.
     */
    public static int m_baseCol = 1;
    /** maximum default size for row.
     */
    public static int m_maxRow = 50;
    /** maximum default size for columns.
     */
    public static int m_maxCol = 50;
    
    //Newly added - JJ
    private FileHistory fileHist = null;
    
    /** UMSpreadSheet Public constructor to create a new spreadsheet.
     */
    public UMSpreadSheet() {
        super("University of Maryland CMSC435 Spreadsheet");
        
        // serey location of dimension
        x = m_config.getInt("X");
        y = m_config.getInt("Y");
        System.out.println("x is " + x );
        System.out.println("y is " + y );
        
        
        Dimension scrdim = getToolkit().getScreenSize();
        
        // make sure the point is valid
        if (x >= 0 && y >= 0 && x < (int)scrdim.getWidth() && y < (int)scrdim.getHeight())
            setLocation(x, y);
        
        setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
        // get content pane
        m_container = this.getContentPane();
        m_container.setLayout(new BorderLayout());
        
        // Create a pane for input bar and toolbar
        m_inputPane = new JPanel();
        m_inputPane.setLayout(new BorderLayout());
        
        //m_inputBar = new JTextField();
        // create the table
        m_table = new JTable(m_maxRow+1, m_maxCol+1);
        m_table.setGridColor(Color.lightGray);
        
        // create the Function combobox in the table
        //Start
        java.awt.GraphicsEnvironment ge =
        java.awt.GraphicsEnvironment.getLocalGraphicsEnvironment();
        m_functionList = new JComboBox();
        m_functionList.addItem("[Automatic]");
        m_functionList.addItem("Sum");
        m_functionList.addItem("Average");
        m_functionList.addItem("Max");
        m_functionList.addItem("Min");
        m_functionList.addItem("Square Root");
        m_functionList.addItem("Sine");
        m_functionList.addItem("Count");
        
        m_functionList.setBackground(Color.white);
        m_functionList.setSelectedIndex(0);
        m_functionList.addItemListener( new ItemListener() {
            public void itemStateChanged( ItemEvent e ) {
                if( e.getStateChange() == ItemEvent.SELECTED ) {
                    String s = (String)e.getItem();
                    
                    if( s.equals("Sum") )
                        m_functionsMenuOp.sumOp();
                    else if( s.equals("Average") )
                        m_functionsMenuOp.averageOp();
                    else if( s.equals("Max") )
                        m_functionsMenuOp.maxOp();
                    else if( s.equals("Min") )
                        m_functionsMenuOp.minOp();
                    else if( s.equals("Square Root") )
                        m_functionsMenuOp.sqrtOp();
                    else if( s.equals("Sine") )
                        m_functionsMenuOp.sineOp();
                    else if( s.equals("Count") )
                        m_functionsMenuOp.countOp();
                    
                    m_functionList.setSelectedIndex(0);
                }
            }
        });
        //End
        
        
        
        // createS the Fonts combobox in the table
        //java.awt.GraphicsEnvironment ge = java.awt.GraphicsEnvironment.getLocalGraphicsEnvironment();
        ge = java.awt.GraphicsEnvironment.getLocalGraphicsEnvironment();
        String[] fontNames = ge.getAvailableFontFamilyNames();
        
        m_fontList = new JComboBox();
        m_fontList.addItem("[Automatic]");
        for (int i = 0; i < fontNames.length; i++)
            m_fontList.addItem(fontNames[i]);
        
        m_fontList.setBackground(Color.white);
        m_fontList.setSelectedIndex(2);
        m_fontList.addItemListener(new ItemListener() {
            public void itemStateChanged(ItemEvent e) {
                if (e.getStateChange() == ItemEvent.SELECTED)
                    m_formatMenuOp.setFontOp(e.getItem());
            }
        });
        
        // creates list of font sizes
        String [] fontSizes = { "-", "8", "9", "10", "11", "12", "14", "16", "18",
        "20", "24", "26", "28" ,"36", "48", "72"};
        
        m_fontSizeList = new JComboBox(fontSizes);
        m_fontSizeList.setBackground(Color.white);
        m_fontSizeList.setSelectedIndex(5);
        m_fontSizeList.addItemListener(new ItemListener() {
            public void itemStateChanged(ItemEvent e) {
                if (e.getStateChange() == ItemEvent.SELECTED)
                    m_formatMenuOp.setFontSizeOp(e.getItem());
            }
        });
        // initialize table to default fonts and font sizes
        try {
            m_table.setFont(new Font(fontNames[1], Font.PLAIN, Integer.parseInt(fontSizes[5])));
        } catch (Exception fontE) {
            m_table.setFont(new Font(fontNames[1], Font.PLAIN, 12));
        }
        
        // Sets up fileMenu
        m_fileMenu = new JMenu("File", true);
        m_fileMenu.setMnemonic(KeyEvent.VK_F);
        
        // New
        JMenuItem menuItem = new JMenuItem("New...");
        menuItem.setMnemonic(KeyEvent.VK_N);
        menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_N, ActionEvent.CTRL_MASK));
        menuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                m_fileMenuOp.newFile();
            }
        });
        m_fileMenu.add(menuItem);
        
        // Open
        menuItem = new JMenuItem("Open...");
        menuItem.setMnemonic(KeyEvent.VK_O);
        menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O, ActionEvent.CTRL_MASK));
        menuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                int choice = 2;
                if( m_tableModel.isModified() )
                    choice = m_fileMenuOp.queryForSave();
                if( choice == 0 )
                    m_fileMenuOp.saveFile();
                
                m_fileMenuOp.openFile();
                
            }
        });
        m_fileMenu.add(menuItem);
        m_fileMenu.addSeparator();
        
        // Save
        menuItem = new JMenuItem("Save");
        menuItem.setMnemonic(KeyEvent.VK_S);
        menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, ActionEvent.CTRL_MASK));
        menuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                m_fileMenuOp.saveFile();
            }
        });
        m_fileMenu.add(menuItem);
        
        // Save As
        menuItem = new JMenuItem("Save As...");
        menuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                m_fileMenuOp.saveAsFile();
            }
        });
        m_fileMenu.add(menuItem);
        m_fileMenu.addSeparator();
        
        // Print
        menuItem = new JMenuItem("Print...");
        menuItem.setMnemonic(KeyEvent.VK_P);
        menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_P, ActionEvent.CTRL_MASK));
        menuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                m_fileMenuOp.printFile();
            }
        });
        m_fileMenu.add(menuItem);
        
        // Exit, may need to fix
        m_fileMenu.addSeparator();
        menuItem = new JMenuItem("Exit");
        menuItem.setMnemonic(KeyEvent.VK_E);
        menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_E, ActionEvent.CTRL_MASK));
        menuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                //Newly Added - JJ
                fileHist.saveFileHistory(new File("recentFileHistory.log"));
                m_fileMenuOp.exitFile(2);
            }
        });
        m_fileMenu.add(menuItem);
        
        //AMAR
        //Set up Graphs Menu
        m_graphsMenu = new JMenu( "Graphs", true );
        m_graphsMenu.setMnemonic(KeyEvent.VK_G);
        menuItem = new JMenuItem( "Pie" );
        menuItem.addActionListener( new ActionListener()  {
            public void actionPerformed( ActionEvent evt ) {
                m_graphsMenuOp.pieGraph();
            }
        });
        m_graphsMenu.add( menuItem );
        
        menuItem = new JMenuItem( "Column" );
        menuItem.addActionListener( new ActionListener()  {
            public void actionPerformed( ActionEvent evt ) {
                m_graphsMenuOp.columnGraph();
            }
        });
        m_graphsMenu.add( menuItem );
        
        menuItem = new JMenuItem( "Line" );
        menuItem.addActionListener( new ActionListener()  {
            public void actionPerformed( ActionEvent evt ) {
                m_graphsMenuOp.lineGraph();
            }
        });
        m_graphsMenu.add( menuItem );
        
        menuItem = new JMenuItem( "Row" );
        menuItem.addActionListener( new ActionListener()  {
            public void actionPerformed( ActionEvent evt ) {
                m_graphsMenuOp.rowGraph();
            }
        });
        m_graphsMenu.add( menuItem );
        //AMAR
        
        //FileHistory - Newly Added - JJ
        File iniFile = new File("recentFileHistory.log");
        fileHist = new FileHistory(this, iniFile, m_fileMenu);
        
        
        //Set up Edit menu
        m_editMenu = new JMenu("Edit", true);
        m_editMenu.setMnemonic(KeyEvent.VK_E);
        
        // ADDED BY AL UNDO
        menuItem = new JMenuItem("Undo");
        menuItem.setMnemonic(KeyEvent.VK_Z);
        menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Z, ActionEvent.CTRL_MASK));
        menuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                System.out.println("undo actionperformed");
                m_editMenuOp.undo_MenuItemActionPerformed();
                System.out.println("undo action performed exited");
            }
        });
        m_editMenu.add(menuItem);
        
        // ADDED BY AL REDO
        menuItem = new JMenuItem("Redo");
        menuItem.setMnemonic(KeyEvent.VK_Y);
        menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Y, ActionEvent.CTRL_MASK));
        menuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                m_editMenuOp.redoActionPerformed();
            }
        });
        m_editMenu.add(menuItem);
        m_editMenu.addSeparator();
        
        // Cut
        menuItem = new JMenuItem("Cut");
        menuItem.setMnemonic(KeyEvent.VK_X);
        menuItem.setAccelerator(KeyStroke.getKeyStroke( KeyEvent.VK_X, ActionEvent.CTRL_MASK));
        menuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                m_editMenuOp.cutOp();
            }
        });
        m_editMenu.add(menuItem);
        // Copy
        menuItem = new JMenuItem("Copy");
        menuItem.setMnemonic(KeyEvent.VK_C);
        menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_C, ActionEvent.CTRL_MASK));
        menuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                m_editMenuOp.copyOp();
            }
        });
        m_editMenu.add(menuItem);
        // Paste
        menuItem = new JMenuItem("Paste");
        menuItem.setMnemonic(KeyEvent.VK_V);
        menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_V, ActionEvent.CTRL_MASK));
        menuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                m_editMenuOp.pasteOp();
            }
        });
        m_editMenu.add(menuItem);

        // Paste Special
        menuItem = new JMenuItem("Paste Speical");
        menuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                m_editMenuOp.pasteSpOp();
            }
        });
        m_editMenu.add(menuItem);
        m_editMenu.addSeparator();
                
        // Clear
        menuItem = new JMenuItem("Clear");
        menuItem.setMnemonic(KeyEvent.VK_DELETE);
        menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_DELETE, 0, true));
        menuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                m_editMenuOp.clearOp();
            }
        });
        m_editMenu.add(menuItem);
        
        
        // Find
        m_editMenu.addSeparator();
        menuItem = new JMenuItem("Find...");
        menuItem.setMnemonic(KeyEvent.VK_F);
        menuItem.setAccelerator(KeyStroke.getKeyStroke
        (KeyEvent.VK_F, ActionEvent.CTRL_MASK));
        menuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                m_editMenuOp.find(true);
            }
        });
        m_editMenu.add(menuItem);
        
        
        
        //Set up Format Menu
        m_formatMenu = new JMenu("Format", true);
        m_formatMenu.setMnemonic(KeyEvent.VK_F);
        
        //set up tableMenu actions
        insertMenu = new JMenu("Insert");
        insertMenu.setMnemonic(KeyEvent.VK_I);
        
        menuItem = new JMenuItem("Row");
        menuItem.setMnemonic(KeyEvent.VK_R);
        menuItem.setAccelerator(KeyStroke.getKeyStroke
        (KeyEvent.VK_INSERT, ActionEvent.CTRL_MASK));
        menuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                m_formatMenuOp.insert(true);
            }
        });
        insertMenu.add(menuItem);
        
        menuItem = new JMenuItem("Column");
        menuItem.setMnemonic(KeyEvent.VK_C);
        menuItem.setAccelerator(KeyStroke.getKeyStroke
        (KeyEvent.VK_INSERT,
        ActionEvent.CTRL_MASK|
        ActionEvent.SHIFT_MASK));
        menuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                m_formatMenuOp.insert(false);
            }
        });
        insertMenu.add(menuItem);
        m_formatMenu.add(insertMenu);
        
        //set up Delete actions
        deleteMenu = new JMenu("Delete");
        deleteMenu.setMnemonic(KeyEvent.VK_D);
        menuItem = new JMenuItem("Row");
        menuItem.setMnemonic(KeyEvent.VK_R);
        menuItem.setAccelerator(KeyStroke.getKeyStroke
        (KeyEvent.VK_DELETE, ActionEvent.CTRL_MASK));
        menuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                //notYetImplemented();
                m_formatMenuOp.remove(true);
            }
        });
        deleteMenu.add(menuItem);
        menuItem = new JMenuItem("Column");
        menuItem.setMnemonic(KeyEvent.VK_C);
        menuItem.setAccelerator(KeyStroke.getKeyStroke
        (KeyEvent.VK_DELETE,
        ActionEvent.CTRL_MASK|
        ActionEvent.SHIFT_MASK));
        menuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                //notYetImplemented();
                m_formatMenuOp.remove(false);
            }
        });
        deleteMenu.add(menuItem);
        m_formatMenu.add(deleteMenu);
        
        m_formatMenu.addSeparator();
        
        // Set column width
        menuItem = new JMenuItem("Column Width");
        menuItem.setMnemonic(KeyEvent.VK_W);
        menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_W, ActionEvent.CTRL_MASK));
        menuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                m_formatMenuOp.columnWidthOp();
            }
        });
        m_formatMenu.add(menuItem);
        // Row Height
        menuItem = new JMenuItem("Row Height");
        menuItem.setMnemonic(KeyEvent.VK_H);
        menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_H, ActionEvent.CTRL_MASK));
        menuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                m_formatMenuOp.rowHeightOp();
            }
        });
        m_formatMenu.add(menuItem);
        // ADDED BY AL AUTOFIT
        menuItem = new JMenuItem("Autofit");
        menuItem.setMnemonic(KeyEvent.VK_F);
        menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F, ActionEvent.CTRL_MASK));
        menuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                m_formatMenuOp.columnAutofitOp();
            }
        });
        m_formatMenu.add(menuItem);
        // Format Cells
        m_formatMenu.addSeparator();
        menuItem = new JMenuItem("Format Cells...");
        menuItem.setMnemonic(KeyEvent.VK_R);
        menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_R, ActionEvent.CTRL_MASK));
        menuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                m_formatMenuOp.formatCellsOp();
            }
        });
        m_formatMenu.add(menuItem);
        
        //Set up Functions Menu
        m_functionsMenu = new JMenu("Functions", true);
        m_functionsMenu.setMnemonic(KeyEvent.VK_U);
        
        // Sum
        menuItem = new JMenuItem("Sum");
        menuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                m_functionsMenuOp.sumOp();
            }
        });
        m_functionsMenu.add(menuItem);
        
        // Average
        menuItem = new JMenuItem("Average");
        menuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                m_functionsMenuOp.averageOp();
            }
        });
        m_functionsMenu.add(menuItem);
        // Max
        menuItem = new JMenuItem("Max");
        menuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                m_functionsMenuOp.maxOp();
            }
        });
        m_functionsMenu.add(menuItem);
        
        // Min
        menuItem = new JMenuItem("Min");
        menuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                m_functionsMenuOp.minOp();
            }
        });
        m_functionsMenu.add(menuItem);
        
        // Square Root
        menuItem = new JMenuItem("Square Root");
        menuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                m_functionsMenuOp.sqrtOp();
            }
        });
        m_functionsMenu.add(menuItem);
        
        // Sine
        menuItem = new JMenuItem("Sine");
        menuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                m_functionsMenuOp.sineOp();
            }
        });
        m_functionsMenu.add(menuItem);
        
        // Count
        menuItem = new JMenuItem("Count");
        menuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                m_functionsMenuOp.countOp();
            }
        });
        m_functionsMenu.add(menuItem);
        
        
        /***** Serey's new Code for about Button
         **/
        // Set up Help Menu
        m_helpMenu = new JMenu("Help", true);
        m_helpMenu.setMnemonic(KeyEvent.VK_H);
        
        
        // User Manual - Newly Added - JJ
        menuItem = new JMenuItem("User Manual");
        menuItem.setMnemonic(KeyEvent.VK_M);
        menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_M, ActionEvent.CTRL_MASK));
        menuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                m_helpMenuOp.displayUserManual();
            }});
            m_helpMenu.add(menuItem);
            
            // About
            menuItem = new JMenuItem("About TerpSpreedsheet");
            menuItem.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent evt) {
                    m_helpMenuOp.aboutBox();
                }
            });
            m_helpMenu.add(menuItem);
            
            
            
            /**** end of serey's new code for about button */
            
            
            
            
            //*******Popup Menu Method******
            m_popup = new PopupMenu("Edit");
            MenuItem mi;
            
            mi = new MenuItem("Format Cells...");
            mi.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent evt) {
                    m_formatMenuOp.formatCellsOp();
                    m_table.requestDefaultFocus();
                }
            });
            m_popup.add(mi);
            m_popup.addSeparator();
            
            mi = new MenuItem("Cut");
            mi.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent evt) {
                    m_editMenuOp.cutOp();
                    m_table.requestDefaultFocus();
                }
            });
            m_popup.add(mi);
            
            mi = new MenuItem("Copy");
            mi.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent evt) {
                    m_editMenuOp.copyOp();
                    m_table.requestDefaultFocus();
                }
            });
            m_popup.add(mi);
            
            mi = new MenuItem("Paste");
            mi.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent evt) {
                    m_editMenuOp.pasteOp();
                    m_table.requestDefaultFocus();
                }
            });
            m_popup.add(mi);
            m_popup.addSeparator();
            
            mi = new MenuItem("Clear");
            mi.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent evt) {
                    m_editMenuOp.clearOp();
                    m_table.requestDefaultFocus();
                }
            });
            m_popup.add(mi);
            
            m_table.add(m_popup);
            m_table.addMouseListener(this);
            
            // set up menu bar and menus
            m_menuBar = new JMenuBar();
            m_menuBar.add(m_fileMenu);
            m_menuBar.add(m_editMenu);
            m_menuBar.add(m_formatMenu);
            m_menuBar.add(m_functionsMenu);
            m_menuBar.add(m_graphsMenu); //AMar
            // serey s
            m_menuBar.add(m_helpMenu);
            
            
            // button initializations
            JButton newButton = new JButton(newIcon);
            newButton.setToolTipText("New");
            newButton.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent evt) {
                    m_fileMenuOp.newFile();
                    m_table.requestFocus();
                }
            });
            
            JButton openButton = new JButton(openIcon);
            openButton.setToolTipText("Open");
            openButton.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent evt) {
                    int choice = 2;
                    if( m_tableModel.isModified() )
                        choice = m_fileMenuOp.queryForSave();
                    if( choice == 0 )
                        m_fileMenuOp.saveFile();
                                        
                    m_fileMenuOp.openFile();
                    m_table.requestFocus();
                }
            });

            JButton saveButton = new JButton(saveIcon);
            saveButton.setToolTipText("Save");
            saveButton.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent evt) {
                    m_fileMenuOp.saveFile();
                    m_table.requestFocus();
                }
            });
            
            JButton printButton = new JButton(printIcon);
            printButton.setToolTipText("Print");
            printButton.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent evt) {
                    m_fileMenuOp.printFile();
                    m_table.requestFocus();
                }
            });
            
            JButton cutButton = new JButton(cutIcon);
            cutButton.setToolTipText("Cut");
            cutButton.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent evt) {
                    m_editMenuOp.cutOp();
                    m_table.requestFocus();
                }
            });
            
            JButton copyButton = new JButton(copyIcon);
            copyButton.setToolTipText("Copy");
            copyButton.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent evt) {
                    m_editMenuOp.copyOp();
                    m_table.requestFocus();
                }
            });
            
            JButton pasteButton = new JButton(pasteIcon);
            pasteButton.setToolTipText("Paste");
            pasteButton.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent evt) {
                    m_editMenuOp.pasteOp();
                    m_table.requestFocus();
                }
            });
            /* serey said remove bold & italics
            italicButton = new JToggleButton(italicIcon);
            italicButton.setToolTipText("Italic");
            italicButton.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent evt) {
                    m_formatMenuOp.setItalicOp(italicButton.isSelected());
                    m_table.requestFocus();
                }
            });
            
            boldButton = new JToggleButton(boldIcon);
            boldButton.setToolTipText("Bold");
            boldButton.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent evt) {
                    m_formatMenuOp.setBoldOp(boldButton.isSelected());
                    m_table.requestFocus();
                }
            });
            */
            // format buttond serey
            JButton formatButton = new JButton( formatIcon );
            formatButton.setToolTipText("Format Button");
            formatButton.addActionListener( new ActionListener() {
                public void actionPerformed( ActionEvent evt ) {
                    m_formatMenuOp.formatCellsOp();
                    m_table.requestFocus();
                }
            });
            
            
            
            
            //=============================== ADDED BY AL formula bar
            formulaField = new JTextField();
            formulaField.setPreferredSize(new Dimension(300, 25));
            formulaField.setMaximumSize(new Dimension(500, 25));
            formulaField.setFont(new Font("DialogInput", Font.PLAIN, 12));
            formulaField.setMinimumSize(new Dimension(100, 25));
            formulaField.addMouseListener(this);
            formulaField.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    System.out.println("formula bar");
                    getFormulaBarInput();
                }
            });
            
            // initilize the undo.redo system inside constructor ADDED BY AL
            undoSupport.addUndoableEditListener(new UndoAdapter());
            refreshUndoRedo();
            
            //  Set up toolbar
            //  Toolbar can be customized in the configuration file.
            m_toolBar = new JToolBar();
            m_toolBar.setFloatable(true); // CHANGED BY AL FROM FALSE -> TRUE
            
            m_toolBar.add(newButton);
            m_toolBar.add(openButton);
            m_toolBar.add(saveButton);
            m_toolBar.addSeparator();
            m_toolBar.add(printButton);
            m_toolBar.addSeparator();
            m_toolBar.add(cutButton);
            m_toolBar.add(copyButton);
            m_toolBar.add(pasteButton);
            m_toolBar.addSeparator();
            
            //serey new code
            m_toolBar.add(formatButton);
            m_toolBar.addSeparator();
            
            // serey new code
            JPanel functionPanel = new JPanel();
            functionPanel.add( m_functionList );
            m_toolBar.add(functionPanel);
            m_toolBar.addSeparator();
            
            
            JPanel fontPanel = new JPanel();
            fontPanel.add(m_fontList);
            fontPanel.add(m_fontSizeList);
            
            m_toolBar.add(fontPanel);
            m_toolBar.addSeparator();
            // since we're not using italics anymore: m_toolBar.add(italicButton);
            // since we're not using bold anymore: m_toolBar.add(boldButton);
            
            newTableModel(m_maxCol+1, m_maxRow+1);
            // set window pos and size
            //int w = 600;
            //int h = 400;
            // set window pos and size serey
            w = m_config.getInt("WIDTH");
            h = m_config.getInt("HEIGHT");
            
            System.out.println("w is " + w );
            System.out.println("h is " + h );
            
            
            if (w >= 0 && h >= 0)
                m_table.setPreferredScrollableViewportSize(new Dimension(w, h));
            
            // init Operations objects
            // Change later to save clipboard info
            m_editMenuOp = new EditMenuOp(this);
            m_fileMenuOp = new FileMenuOp(this);
            m_formatMenuOp = new FormatMenuOp(this);
            m_functionsMenuOp = new FunctionsMenuOp(this);
            m_graphsMenuOp = new GraphsMenuOp( this ); //AMAR
            
            // serey
            m_helpMenuOp = new HelpMenuOp(this);
            
            // clobber resizing of all columns
            m_table.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
            
            // set table editor and renderer to custom ones DO THIS LATER
            m_table.setDefaultRenderer(UMCell.class, new UMCellRenderer());
            m_table.setDefaultEditor(UMCell.class, new UMCellEditor(new JTextField()));
            
            // set selection mode for contiguous intervals
            m_table.setSelectionMode(ListSelectionModel.SINGLE_INTERVAL_SELECTION);
            m_table.setCellSelectionEnabled(true);
            
            // we don't allow reordering
            m_table.getTableHeader().setReorderingAllowed(false);
            m_table.getTableHeader().addMouseListener(new HeaderMouseAdapter());
            
            // create selection models
            m_rowSelectionModel = m_table.getSelectionModel();
            m_columnSelectionModel = m_table.getColumnModel().getSelectionModel();
            
            // add selection listeners to the selection models
            m_rowSelectionModel.addListSelectionListener(this);
            m_columnSelectionModel.addListSelectionListener(this);
            
            // set menu bar
            setJMenuBar(m_menuBar);
            // Set toolbar and inputBar on one Pane
            m_inputPane.add(m_toolBar, BorderLayout.NORTH);
            //m_container.add(m_inputPane, BorderLayout.NORTH); DELETED BY AL
            m_inputPane.add(formulaField, BorderLayout.SOUTH);  // ADDED BY AL
            m_container.add(m_inputPane, BorderLayout.NORTH); // ADDED BY AL FOR FORMULABAR
            m_scrollPane = new JScrollPane(m_table, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
            m_container.add(m_scrollPane, BorderLayout.CENTER);
            
            // add window exit listener
            addWindowListener(new WindowAdapter() {
                public void windowOpened(WindowEvent evt) {
                    m_table.requestFocus();
                }
                
                public void windowClosing(WindowEvent evt) {
                    exit();
                }
            });
            
            // initial selection
            resetSelection();
            
            m_table.setRequestFocusEnabled(true);
            m_menuBar.setRequestFocusEnabled(false);
            m_toolBar.setRequestFocusEnabled(false);
            formulaField.setRequestFocusEnabled(true); //ADDED BY AL
            m_table.requestFocus();
            formulaField.requestFocus();// ADDED BY AL
            
            pack();
    }
    
    
    /** serey
     * Check the find next menu/button state
     * Enable only when the user has searched once
     */
    public void checkFindNextState() {
        findNextMenuItem.setEnabled(m_editMenuOp.hasFindValue());
    }
    
    /** Sets the width of the columns.
     *
     * @param colWidth specifics the width of the columns
     */
    public void setColumnWidth(int colWidth) {
        if (colWidth > 0)
            for (int i = m_baseCol; i < m_tableModel.getColumnCount(); i ++) {
                TableColumn column = m_table.getColumnModel().getColumn(i);
                column.setMinWidth(colWidth);
                column.setPreferredWidth(colWidth);
            }
    }
    
    /** Creates blank SharpTableModel object with the specified number of
     * rows and columns.  The table is set to this table model to update screen.
     *
     * @param rows number of rows in new table model
     * @param cols number of columns in new table model
     */
    public void newTableModel(int rows, int cols) {
        m_tableModel = new UMTableModel(this, rows, cols);
        m_table.setModel(m_tableModel);
        
        setBaseColumnWidth();
        setColumnWidth(80);
        
        m_tableModel.setModified(false);
        resetSelection();
        m_table.requestFocus();
    }
    
    /** Opens a spreadsheet from a file, used only in main.
     *
     * @param name filename
     */
    public void openInitFile(String name) {
        m_fileMenuOp.openTableModel(new File(name));
    }
    
    /** Exits the application.
     */
    public void exit() {
        // serey
        m_config.setInt("X", getX());
        m_config.setInt("Y", getY());
        
        Dimension dim = m_scrollPane.getViewport().getExtentSize();
        
        m_config.setInt("WIDTH", (int)dim.getWidth());
        m_config.setInt("HEIGHT", (int)dim.getHeight());
        m_config.save(); // saves the file
        //Newly Added - JJ
        fileHist.saveFileHistory(new File("recentFileHistory.log"));
        
        m_fileMenuOp.exitFile(1);
    }
    
    /** Checks the save menu button state.  Enabled only when the current file has
     * been modified.
     */
    public void checkSaveState() {
        boolean modified = m_tableModel.isModified();
        //saveMenuItem.setEnabled(modified);
        //saveButton.setEnabled(modified);
    }
    
    /** Initializes the width of the columns.
     */
    public void setBaseColumnWidth() {
        // resize first column
        if (m_baseCol > 0) {
            TableColumn firstColumn = m_table.getColumnModel().getColumn(m_baseCol - 1);
            int firstColWidth = 25;
            if (firstColWidth > 0) {
                firstColumn.setMinWidth(firstColWidth);
                firstColumn.setPreferredWidth(firstColWidth);
            }
        }
    }
    
    /** Starts up the spreadsheet as a stand alone aplication.
     *
     * @param args file name
     */
    public static void main(String args[]) {
        
        // serey modified this
        // read configuration file
        System.out.println("LOADING CONFIG FILE!!!!");
        //m_config = new Config("c:\\final\\umspreadsheet.ini");
        m_config = new Config("umspreadsheet.ini");
        
        
        
        m_config.load();
        Debug.setDebug(false);
        
        UMSpreadSheet spreadsheet = new UMSpreadSheet();
        
        // Newly Added - JJ
        Splash mySplash = new Splash(spreadsheet, "TerpSpreadsheet_Splash.jpg");
        // dummy delay so we can see the Splash! - JJ
        for(int i = 0; i < 3000; i++) {
            //System.out.println(i) ;
        }
        
        spreadsheet.m_isStandAlone = true;
        
        if (args.length > 0){
            //System.out.println("----> "+args[0]);
            spreadsheet.openInitFile(args[0]);
        }
        
        Formula.registerFunctions();
        
        // Newly moved - JJ
        spreadsheet.show();
    }
    
    /** Should be used to updated the operations if table model changes.
     */
    public void updateOps() {
        m_formatMenuOp.updateTableModel();
    }
    
    /** Static function to help load spreadsheet images.
     *
     * @param name image name
     * @return location of the image
     */
    public static ImageIcon getImageIcon(String name) {
        URL url = ClassLoader.getSystemResource(name);
        if (url == null) {
            System.out.println("image " +name+ " not found");
            return null;
        }
        return new ImageIcon(url);
    }
    
    /** INTERFACE: ListSelectionListener
     * Sets the selection or the table, determines that the shortcut formating gui
     * is updated for the new selection.
     *
     * @param e current event
     */
    public void valueChanged(ListSelectionEvent e) {
        m_table.requestFocus();
        
        // Ignore extra messages
        if (e.getValueIsAdjusting()) return;
        
        // Get event source
        ListSelectionModel lsm = (ListSelectionModel)e.getSource();
        
        if (lsm.isSelectionEmpty()) {
            // always set selection
            m_table.setRowSelectionInterval(m_baseRow, m_baseRow);
            m_table.setColumnSelectionInterval(m_baseCol, m_baseCol);
        } else {
            // Get selected column
            if (m_table.getSelectedColumn() < m_baseCol) {
                int columnCount = m_table.getColumnCount();
                m_table.setColumnSelectionInterval(m_baseCol, columnCount - 1);
                m_table.removeColumnSelectionInterval(m_baseRow,m_baseRow);
            }
        }
        
        // Get the state of cells, and be ready to reset the gue
        CellRange cells = m_tableModel.getSelection();
        boolean italic = m_formatMenuOp.isItalic(cells);
        boolean bold = m_formatMenuOp.isBold(cells);
        String fontSize = m_formatMenuOp.getFontSize(cells);
        String fontName = m_formatMenuOp.getFontName(cells);
        
        // no use bold anymore: boldButton.setSelected(bold);
        // no use italics anymore: italicButton.setSelected(italic);
        
        // Update the formulaField with the contents of the newly selected cell
        CellPoint cp;
        cp = cells.getminCorner();
        formulaField.setText(m_tableModel.getCellAt(cp.getRow(), cp.getCol()).toString());
        
        
        // Declare final stuff to use in the later funciton
        final String fFontName = fontName;
        final String fFontSize = fontSize;
        
        // Set this on event dispach thread
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                for (int i = 0; i < m_fontList.getItemCount(); i++) {
                    //System.out.println(
                    //   (String)m_fontList.getItemAt(i) +
                    //   " " + fFontName);
                    
                    if (((String)m_fontList.getItemAt(i)).compareTo(fFontName) == 0) {
                        m_fontList.setSelectedIndex(i);
                        break;
                    }
                }
                for (int i = 0; i < m_fontSizeList.getItemCount(); i++) {
                    if (((String)m_fontSizeList.getItemAt(i)).compareTo(fFontSize) == 0) {
                        m_fontSizeList.setSelectedIndex(i);
                        break;
                    }
                }
            }});
    }
    
    /** Resets the selection to the first row and first column.
     */
    public void resetSelection() {
        m_table.setRowSelectionInterval(m_baseRow, m_baseRow);
        m_table.setColumnSelectionInterval(m_baseCol, m_baseCol);
    }
    
    // INTEFACE: MouseAdapter
    /** indicates the current action being performed.
     *
     * @param e current action
     */
    public void actionPerformed(ActionEvent e) {}
    /** unused.
     *
     * @param e none
     */
    public void mouseEntered(MouseEvent e){}
    /** unused.
     *
     * @param e none
     */
    public void mouseExited(MouseEvent e){}
    /** unused.
     *
     * @param e none
     */
    public void mouseReleased(MouseEvent e){}
    /** unused.
     *
     * @param e none
     */
    public void mousePressed(MouseEvent e) {
    }
    
    /** Displays the popup menu when clicked.
     *
     * @param e mouse click event
     */
    public void mouseClicked(MouseEvent e) {
        if (e.getModifiers() == InputEvent.BUTTON3_MASK)
            m_popup.show(e.getComponent(), e.getX(), e.getY());
    }
    
    /** This class is the mouse listener for the table and is responsible for
     * all of the mouse actions.
     */
    class HeaderMouseAdapter extends MouseAdapter {
        
        /** Determines if the mouse action has occured.
         * @param e mouse click event
         */
        public void mouseClicked(MouseEvent e) {
            TableColumnModel colModel = m_table.getColumnModel();
            int col = colModel.getColumn(colModel.getColumnIndexAtX(e.getX())).getModelIndex();
            
            int rowCount = m_table.getRowCount();
            m_table.setRowSelectionInterval(m_baseRow, rowCount - 1);
            
            if (col < m_baseCol)
                m_table.setColumnSelectionInterval(m_baseCol, m_table.getColumnCount()-1);
            else
                m_table.setColumnSelectionInterval(col, col);
        }
    }
    
    /** INTEFACE: TerpOffice
     * Used to exit the application without invoking system.exit.
     */
    public void close() {
        if (alive())
            exit();
        //TerpOffice.removeApps(this);
    }
    
    /** Starts up the spreadsheet within an office.
     *
     * @param file configuration file name
     */
    public void launch(File file) {
        // read configuration file
        //m_config = new Config("umspreadsheet.ini");
        
        Debug.setDebug(false);
        
        //UMSpreadSheet spreadsheet = new UMSpreadSheet();
        this.show();
        if (file != null)
            m_fileMenuOp.openTableModel(file);
        Formula.registerFunctions();
    }
    
    /** Returns true if spreadsheet window has not been disposed yet.
     *
     * @return determines if spreadsheet is available
     */
    public boolean alive() {
        return isDisplayable();
    }

    /** Starts to print the file if need be.
     *
     * @returns true if file was able to be printed
     */
    public boolean TerpPrint(File file) {
        m_fileMenuOp.setFile( file );
	  m_fileMenuOp.printData();
	  return true;
    }    
    
    private void getFormulaBarInput() {
        //debugMessage("WebSheetApplet.getFormulaBarInput()\n");
        int selectedRow = m_table.getSelectedRow();
        int selectedColumn = m_table.getSelectedColumn();
        
        int VjsbValue = m_scrollPane.getVerticalScrollBar().getValue();
        int HjsbValue = m_scrollPane.getHorizontalScrollBar().getValue();
        
        if (selectedRow >= 0 && selectedColumn >= 0) {
            m_tableModel.setValueAt(formulaField.getText(), selectedRow, selectedColumn);
            m_table.changeSelection(selectedRow, selectedColumn, false, false);
        }
        
        m_scrollPane.getVerticalScrollBar().setValue(VjsbValue);
        m_scrollPane.getHorizontalScrollBar().setValue(HjsbValue);
        
        formulaField.requestFocus();
    }
    // ADDED BY AL
    /** This method will refresh the edit menu to indicate when a
     * undo or redo can occur.
     *
     */
    public void refreshUndoRedo() {
        System.out.println("refreshundoredo");
        // refresh undo
        m_editMenu.getItem(0).setText(undoManager.getUndoPresentationName());
        m_editMenu.getItem(0).setEnabled(undoManager.canUndo());
        
        // refresh redo
        m_editMenu.getItem(1).setText(undoManager.getRedoPresentationName());
        m_editMenu.getItem(1).setEnabled(undoManager.canRedo());
        System.out.println("refreshed exit");
    }
    
    private class UndoAdapter implements UndoableEditListener {
        /** This adaptor is created to determine if a clear/cut/paste operation
         * has occured so that the undomanager can register it and create
         * the undo/redo functionality.
         *
         * @param evt the event that occured
         */
        public void undoableEditHappened(UndoableEditEvent evt) {
            System.out.println("the adaptor");
            undoManager.addEdit(evt.getEdit());
            refreshUndoRedo();
            System.out.println("the adaptor exited");
        }
    }
    
    /** Method that determines if no cells have been selected.
     *
     * @param s string
     */
    public void noCellsSelected(String s) {
        UMOptionPane.showMessageDialog(this, s + ": No cells selected","Error", JOptionPane.ERROR_MESSAGE, null);
    }
    
    //Newly added - JJ
    /** This method returns the file history log.
     *
     * @return name of the file history log
     */
    public FileHistory getFileHistory() {
        return fileHist;
    }
}